/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.cdm;

import java.io.*;
import java.util.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.CollectionConfiguration;
import org.greenstone.gatherer.collection.Collection;
import org.greenstone.gatherer.collection.CollectionManager;
import org.greenstone.gatherer.msm.ElementWrapper;
import org.greenstone.gatherer.msm.MetadataSetManager;
import org.greenstone.gatherer.msm.MSMUtils;
import org.greenstone.gatherer.util.StaticStrings;
import org.greenstone.gatherer.util.Utility;
import org.w3c.dom.*;

/** This class contains all the details about a single argument that can be passed to this plugin, including option lists if the parameters are restricted.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public class Argument
    implements Comparable, Serializable {
    /** An element of the argument type enumeration. */
    static final public byte ENUM = 0;
    /** An element of the argument type enumeration. */
    static final public byte FLAG = 1;
    /** An element of the argument type enumeration. */
    static final public byte HIERARCHY = 2;
    /** An element of the argument type enumeration. */
    static final public byte INTEGER = 3;
    /** An element of the argument type enumeration. */
    static final public byte LANGUAGE = 4;
    /** An element of the argument type enumeration. */
    static final public byte METADATA = 5;
    /** An element of the argument type enumeration. */
    static final public byte METADATUM = 6;
    /** An element of the argument type enumeration. */
    static final public byte STRING = 7;
    /** <i>true</i> if this argument is required for the applicable script to work properly, <i>false</i> otherwise. */
    private boolean required = false;
    /** The type of this argument. Initially an int, but bytes are cheaper. */
    private byte type = STRING;
    
    /** The maximum value an integer based control can have. */
    private int maximum = Integer.MAX_VALUE;
    /** The minimum value an integer based control can have. */
    private int minimum = Integer.MIN_VALUE;

    private Element element;
    /** If the argument is of type ENUM then this map holds all the various options. Each entry is an &lt;option value&gt; -&gt; &lt;description&gt; mapping. */
    private HashMap list = null;
    /** A default value for parameter-type arguments. May be a Perl pattern. */
    private String default_value = null;
    /** The text description of this argument parsed from the pluginfo output. */
    private String description = null;
    /** The argument flag as it appears in the command. Also used as the unique identifier of an argument. */
    private String name = null;
    /** The plugin that owns this argument, for the purposes of visualising inheritance. */
    private String owner = null;

    /** Default Constructor. */
    public Argument() {
    }

    public Argument(Element element) {
	this.element = element;
    }

    /** Method to add an element to the option list.
     * @param name The name value of the option as a <strong>String</strong>.
     * @param desc The description of this options as a <strong>String</strong>.
     */
    public void addOption(String name, String desc) {
	if(type == ENUM && name != null) {
	    if(desc == null) {
		desc = "";
	    }
	    if(list == null) {
		list = new HashMap();
	    }
	    list.put(name, desc);
	}
    }

    /** Method to compare two arguments for ordering.
     * @param object The argument we are comparing to, as an <strong>Object</strong>.
     * @return An <i>int</i> specifying the argument order, using values as set out in String.
     * @see java.lang.String#compareTo
     * @see org.greenstone.gatherer.cdm.Argument
     */
    public int compareTo(Object object) {
	if(object instanceof Argument) {
	    return getName().compareTo(((Argument)object).getName());
	}
	else {
	    return toString().compareTo(object.toString());
	}
    }

    public Argument copy() {
	Argument copy = new Argument();
	copy.setDefaultValue(default_value);
	copy.setDescription(description);
	copy.setOptions(list);
	copy.setOwner(owner);
	copy.setName(name);
	copy.setRequired(required);
	copy.setType(type);
	return copy;
    }

    /** Method to determine if two arguments are equal.
     * @param object The argument to test against, as an <strong>Object</strong>.
     * @return <i>true</i> if the arguments names match, <i>false</i> otherwise.
     */
    public boolean equals(Object object) {
	return (compareTo(object) == 0);
    }

    /** Method to retrieve the value of default_value.
     * @return A <strong>String</strong> containing the default value.
     */
    public String getDefaultValue() {
	return default_value;
    }

    /** Method to retrieve this arguments description.
     * @return A <strong>String</strong> containing the description.
     */
    public String getDescription() {
	return description;
    }

    /** Method to retrieve the description of a certain list option value.
     * @param key The <strong>String</strong> whose description we are searching for.
     * @return The description of the desired key as a <strong>String</strong> which may be empty if no such key exists.
     */
    public String getDescription(String key) {
	if(list.containsKey(key)) {
	    return (String)list.get(key);
	}
	return "";
    }

    public int getMaximum() {
	return maximum;
    }

    public int getMinimum() {
	return minimum;
    }

    /** Method to retrieve the option list for this argument.
     * @return A <strong>HashMap</strong> containing &lt;option value&gt; -&gt; &lt;description&gt; entries.
     */
    public HashMap getOptions() {
	return list;
    }

    /** Method to retrieve the value of name.
     * @return A <strong>String</strong> containing the argument name.
     */
    public String getName() {
	if(name == null && element != null) {
	    name = element.getAttribute(StaticStrings.NAME_ATTRIBUTE);
	}
	return name;
    }

    /** Retrieve the name of the owner of this argument.
     * @return The owners name as a <strong>String</strong>.
     */
    public String getOwner() {
	return owner;
    }
    
    /** Method to determine the type of this argument.
     * @return An <i>byte</i> specifying the type.
     */
    public byte getType() {
	return type;
    }

    /** Method to retrieve the value of value.
     * @return The value of value as a <strong>String</strong>.
     */
    public String getValue() {
	String value = null;
	// Only assigned arguments have values. 
	if(element != null) {
	    value = MSMUtils.getValue(element);
	    // We may have to retrieve the language dependant string for a MSM Element
	    if(type == METADATUM) {
		ElementWrapper element_wrapper = Gatherer.c_man.getCollection().msm.getElement(value);
		if(element_wrapper != null) {
		    value = element_wrapper.getName();
		}
	    }
	}
	return value;
    }

    /** Retrieve the vector of values.
     * @return An <strong>ArrayList</strong> of values.
     */
    public ArrayList getValues() {
	ArrayList values = new ArrayList();
	// Only assigned arguments have values. 
	if(element != null) {
	    String value = MSMUtils.getValue(element);
	    StringTokenizer tokenizer = new StringTokenizer(value, ",");
	    while(tokenizer.hasMoreTokens()) {
		String token = tokenizer.nextToken();
		if(type == METADATA) {
		    ElementWrapper element_wrapper = Gatherer.c_man.getCollection().msm.getElement(token);
		    if(element_wrapper != null) {
			token = element_wrapper.toString();
		    }
		}
		values.add(token);
	    }
	}
	return values;
    }

    /** Method to determine if this argument has been assigned.
     * @return <i>true</i> if it has, <i>false</i> otherwise.
     */
    public boolean isAssigned() {
	return (element != null && element.getAttribute(StaticStrings.ASSIGNED_ATTRIBUTE).equals(StaticStrings.TRUE_STR));
    }

    public boolean isCustomArgument() {
	return (element != null && element.getAttribute(StaticStrings.CUSTOM_ATTRIBUTE).equals(StaticStrings.TRUE_STR));
    }

    /** Method to determine of this argument is required for the associated script to work.
     * @return <i>true</i> if this argument is required, <i>false</i> otherwise.
     */
    public boolean isRequired() {
	return required;
    }

    /** Method to allow for the activation of arguments that might never have their setValue() method called.
     * @param assigned the desired state as a boolean
     */
    public void setAssigned(boolean assigned) {
	if(element != null) {
	    element.setAttribute(StaticStrings.ASSIGNED_ATTRIBUTE, (assigned ? StaticStrings.TRUE_STR : StaticStrings.FALSE_STR));
	}
    }

    public void setCustomArgument(boolean custom) {
	if(element != null) {
	    element.setAttribute(StaticStrings.CUSTOM_ATTRIBUTE, (custom ? StaticStrings.TRUE_STR : StaticStrings.FALSE_STR));
	}
    }

    /** Sets the value of default_value.
     * @param default_value The new value for default_value as a <strong>String</strong>.
     */
    public void setDefaultValue(String default_value) {
	this.default_value = default_value;
    }

    /** Set the value of desc.
     * @param description The new value of desc as a <strong>String</strong>.
     */
    public void setDescription(String description) {
	this.description = description;
    }

    public void setElement(Element element) {
	this.element = element;
    }

    public void setMaximum(int maximum) {
	this.maximum = maximum;
    }

    public void setMinimum(int minimum) {
	this.minimum = minimum;
    }

    /** Set the value of name.
     * @param name The new value of name as a <strong>String</strong>.
     */
    public void setName(String name) {
	this.name = name;
    }

    /** Sets the value of the options list.
     * @param list The new options list as a <strong>HashMap</strong>.
     */
    public void setOptions(HashMap list) {
	this.list = list;
    }

    /** Set the owner of this argument.
     * @param owner The name of the owner of this argument as a <strong>String</strong>.
     */
    public void setOwner(String owner) {
	this.owner = owner;
    }

    /** Set the value of required.
     * @param required The new value of required as a <i>boolean</i>.
     */
    public void setRequired(boolean required) {
	this.required = required;
    }

    /** Set the value of type.
     * @param type The new value of type as an <i>byte</i>.
     */
    public void setType(byte type) {
	this.type = type;
    }

    /** Set the value of type, by matching a type to the given string.
     * @param new_type A <strong>String</strong> which contains the name of a certain argument type.
     */
    public void setType(String new_type) {
	if(new_type.equalsIgnoreCase(StaticStrings.ENUM_STR)) {
	    this.type = ENUM;
	    list = new HashMap();
	}
	else if(new_type.equalsIgnoreCase(StaticStrings.FLAG_STR)) {
	    this.type = FLAG;
	}
	else if(new_type.equalsIgnoreCase(StaticStrings.HIERARCHY_STR)) {
	    this.type = HIERARCHY;
	}
	else if(new_type.equalsIgnoreCase(StaticStrings.INT_STR)) {
	    this.type = INTEGER;
	}
	else if(new_type.equalsIgnoreCase(StaticStrings.LANGUAGE_STR)) {
	    this.type = LANGUAGE;
	}
	else if(new_type.equalsIgnoreCase(StaticStrings.METADATA_TYPE_STR)) {
	    this.type = METADATA;
	}
	else if(new_type.equalsIgnoreCase(StaticStrings.METADATUM_TYPE_STR)) {
	    this.type = METADATUM;
	}
	else {
	    this.type = STRING;
	}
	
    }
    /** Method to set the value of this argument.
     * @param value
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.msm.MSMUtils 
     */
    public void setValue(String value) {
	if(element != null) {
	    MSMUtils.setValue(element, value);
	}
	else {
	    Gatherer.println("Argument.setValue(" + value + ") called on a base Argument.");
	}
    }

    /** Set the values vector to the given values. Currently I just assign the new values, whereas I may later want to implement a deep clone.
     * @param values an ArrayList of values
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.msm.MSMUtils   
     */
    public void setValues(ArrayList values) {
	if(element != null) {
	    StringBuffer value = new StringBuffer();
	    int value_length = values.size();
	    for(int i = 0; i < value_length; i++) {
		value.append(values.get(i));
		value.append(StaticStrings.COMMA_CHARACTER);
	    }
	    value.deleteCharAt(value.length() - 1); // Remove last ','
	    MSMUtils.setValue(element, value.toString());
	}
	else {
	    Gatherer.println("Argument.setValues([" + values.size() + " items]) called on a base Argument.");
	}
    }

    /** Method for translating the data of this class into a string.
     * @return a String containing a fragment of the total arguments string
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.collection.Collection
     * @see org.greenstone.gatherer.collection.CollectionManager
     * @see org.greenstone.gatherer.msm.MetadataSetManager
     * @see org.greenstone.gatherer.msm.MSMUtils
     * @see org.greenstone.gatherer.util.StaticStrings
     */
    public String toString() {
	StringBuffer text = new StringBuffer("-");
	if(element != null) {
	    if(name == null) {
		name = element.getAttribute(StaticStrings.NAME_ATTRIBUTE);
	    }
	    text.append(name);
	    String value = MSMUtils.getValue(element);
	    if(value.length() > 0) {
		text.append(StaticStrings.SPACE_CHARACTER);
		// If the value contains a space, add speech marks
		if(value.indexOf(StaticStrings.SPACE_CHARACTER) != -1) {
		    value = StaticStrings.SPEECH_CHARACTER + value + StaticStrings.SPEECH_CHARACTER;
		}
		// Tokenize the string
		StringTokenizer tokenizer = new StringTokenizer(value, ",");
		while(tokenizer.hasMoreTokens()) {
		    String token = tokenizer.nextToken();
		    if(type == METADATA || type == METADATUM) {
			ElementWrapper element_wrapper = Gatherer.c_man.getCollection().msm.getElement(token);
			if(element_wrapper != null) {
			    text.append(element_wrapper.toString());
			    element_wrapper = null;
			}
			else {
			    text.append(token);
			}
		    }
		    else {
			text.append(token);
		    }
		    token = null;
		    text.append(StaticStrings.COMMA_CHARACTER);
		}
		tokenizer = null;
		text.deleteCharAt(text.length() - 1);
	    }
	    value = null;
	}
	return text.toString();
    }   
}
